package org.jabricks.basegui.examples;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Locale;

import javax.swing.*;

import org.jabricks.basegui.gridp.Gridp;
import org.jabricks.basegui.gridp.IGridp;
import org.jabricks.basegui.gridp.GridpModel;
import org.jabricks.basegui.gridp.IPaginator;
import org.jabricks.basegui.gridp.PaginatorConfig;

public class ExampleGridp extends JFrame implements IPaginator
{
	private static final long serialVersionUID = 1L;
	
	private          IGridp   gridp           = null;
	private PaginatorConfig   pageconf        = null;
	private final       int   RECORDS_ON_PAGE = 8;
	
	private ModuleDAO         dao             = null;

    private JButton           btnAdd          = null;
	private JButton           btnDel          = null;
	private Locale            locale          = null;

	private int              data_idx         = 1;
	private int              added_idx        = 0; 
	
	// Список данных
	private Object[][] milk = {{ "Сметана", "кг", 1.2},
			                   { "Сыр"    , "кг", 0.5},
			                   { "Творог" , "кг", 0.9},
			                   { "Молоко" , "л" , 2.5},
			                   { "Кефир"  , "л" , 0.5},
			                   { "Ряженка", "л" , 0.9}};
	private Object[][] data_add = 
		                       {{ "Йогурт 0.5"  , 4, 55, 220},
			                    { "Йогурт 0.25" , 6, 35, 210},
			                    { "Йогурт 1.0"  , 2, 95, 180},
			                    { "Кефир 0.5"   , 2, 45,  90}};
	// Список колонок
	private String []  tblNames   = {"name", "quantity", "price", "cost"};
	private String []  tblColumns = {"Наименование", "Количество", "Цена", "Стоимость"};
	private Integer[]  tblAlign   = {JLabel.LEFT, JLabel.RIGHT, JLabel.RIGHT, JLabel.RIGHT};
	private Class<?>[] tblClasses = {String.class, Integer.class, Integer.class, Integer.class};

	private String []  milkNames    = {"name", "measure", "quantity"};
	private String []  milkColumns  = {"Наименование", "Ед. измерения", "Количество"};
	private Integer[]  milk_align   = {JLabel.LEFT, JLabel.CENTER, JLabel.RIGHT};
	private Class<?>[] milk_classes = {String.class, String.class, Float.class};
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public ExampleGridp()
	{
		super("Gridp example");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		
		locale = new Locale("ru");
        dao = new ModuleDAO();

		createGrid();
		getContentPane().add(gridp.asWidget(), BorderLayout.CENTER);
		getContentPane().add(createControls(), BorderLayout.SOUTH );
		// Загрузка данных
		gridp.addRecords(loadData());
//		gridp.addRecords(pageconf, loadData());

		setSize(760, 360);
		setVisible(true);
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void createGrid()
	{
		// Создание таблицы
		gridp = new Gridp(this, tblClasses, tblNames, tblColumns);
		gridp.setColumnsAlign(tblAlign);
		gridp.setReorderingAllowed(false);
		gridp.setColumnsWidth(new int[] {220, 60, 60, 60});
		// Определение конфигуратора
		pageconf = gridp.getPaginatorConfig();
		pageconf.setRecords_max(dao.getRecordsCount());
		pageconf.setPages((dao.getRecordsCount() / RECORDS_ON_PAGE) + 1);
		pageconf.setPage(1);
		pageconf.setPage_records(RECORDS_ON_PAGE);
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void changeTableModel()
	{
		if (data_idx == 0) {
			gridp.setModel(tblClasses, tblNames, tblColumns);
			gridp.setColumnsAlign(tblAlign);
			gridp.setColumnsWidth(new int[] {220, 60, 60, 60});

            pageconf.setRecords_max(dao.getRecordsCount());
            pageconf.setPages((dao.getRecordsCount() / RECORDS_ON_PAGE) + 1);
            pageconf.setPage(1);
            pageconf.setPage_records(RECORDS_ON_PAGE);

//          gridp.addRecords(pageconf, loadData());
            gridp.addRecords(loadData());
			data_idx++;
		} else {
			gridp.setModel(milk_classes, milkNames, milkColumns);
			gridp.setColumnsAlign(milk_align);
			gridp.setColumnsWidth(new int[] {240, 110, 90});

            pageconf.setRecords_max(milk.length);
            pageconf.setPages((milk.length / RECORDS_ON_PAGE) + 1);
            pageconf.setPage(1);
            pageconf.setPage_records(RECORDS_ON_PAGE);
					
//			gridp.addRecords(pageconf, milk);
			gridp.addRecords(milk);
			data_idx--;
		}
	}		
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private JPanel createControls()
	{
		JButton btnModel = new JButton("Изменение набора данных");
		btnModel.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				changeTableModel();
				btnAdd.setEnabled(data_idx == 1);
				btnDel.setEnabled(data_idx == 1);
			}
		});

		JButton btnLocale = new JButton("Локализация панели навигации");
		btnLocale.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				changeLocale();
			}
		});

		btnAdd = new JButton("Добавить запись");
		btnAdd.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				addRecord();
			}
		});
		
		btnDel = new JButton("Удалить запись");
		btnDel.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				deleteRecord();
			}
		});
		// Панель кнопоку управления
		JPanel pnlButtons = new JPanel();
		pnlButtons.add(btnModel );
		pnlButtons.add(btnLocale);
		pnlButtons.add(btnAdd   );
		pnlButtons.add(btnDel   );
		
		return pnlButtons;
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private void changeLocale()
	{
        if (locale.getLanguage().equalsIgnoreCase("ru"))
            locale = new Locale("en");
        else
            locale = new Locale("ru");
        gridp.setLocale(locale, null);
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Процедура добавления записи
	 */
	private void addRecord()
	{
		if (added_idx < data_add.length) {
            Object[] obj = data_add[added_idx];
            Object[] record = dao.addRecord(obj);
			gridp.addRecord(record);
			added_idx++;
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Процедура удаления записи из таблицы
	 */
	private void deleteRecord()
	{
		// Удаляемая строка
		int row = gridp.getTable().getSelectedRow();
		if (row >= 0) {
			// Модель данных
			GridpModel model = (GridpModel) gridp.getTable().getModel();
            int id = (Integer) model.getTableData().get(row).elementAt(4);
			dao.deleteRecord(id);

			gridp.delRecord(row);
			
			// Обновление записей в таблице
			gridp.addRecords(loadData());
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	private Object[][] loadData()
	{
		int offs = (pageconf.getPage() - 1) * RECORDS_ON_PAGE;
		return dao.loadData(offs, RECORDS_ON_PAGE);
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Отобразить записи первой страницы
	 */	
	@Override
	public void selectFirst()
	{
		if (data_idx == 0)
			return;
		pageconf.setPage(1);
		gridp.addRecords(loadData());
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Отобразить записи предыдущей страницы
	 */	
	@Override
	public void selectPrev()
	{
		if (data_idx == 0)
			return;
		if (pageconf.getPage() > 1) {
			pageconf.setPage(pageconf.getPage() - 1);
		    gridp.addRecords(loadData());
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Отобразить записи следующей страницы
	 */	
	@Override
	public void selectNext() 
	{
		if (data_idx == 0)
			return;
		if (pageconf.getPage() < pageconf.getPages()) {
			pageconf.setPage(pageconf.getPage() + 1);
		    gridp.addRecords(loadData());
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Отобразить записи последней страницы
	 */	
	@Override
	public void selectLast()
	{
		if (data_idx == 0)
			return;
		if (pageconf.getPage() < pageconf.getPages()) {
			pageconf.setPage(pageconf.getPages());
		    gridp.addRecords(loadData());
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	/**
	 * Отобразить записи страницы page
	 * @param page номер страницы
	 */
	@Override
	public void selectPage(int page)
	{
		if (data_idx == 0)
			return;
		if ((page >= 1) && (page <= pageconf.getPages())) {
			pageconf.setPage(page);
		    gridp.addRecords(loadData());
		}
	}
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	public static void main(String[] args) {
		new ExampleGridp();
	}
}